@@ -7,9 +7,9 @@ from pathlib import Path
 from typing import Any, ClassVar, Dict, List, Optional
 from uuid import uuid1
 from xml.etree.ElementTree import Element, SubElement
-from ctypes import cdll, c_char_p, c_void_p, c_int, c_bool
 
-from .osutil import get_lib_extension, get_platform
+from .logmsg import LogMsg
+from .default_experiment import DefaultExperiment
 from ._version import __version__ as VERSION
 from .enums import Fmi2Type, Fmi2Status, Fmi2Causality, Fmi2Initial, Fmi2Variability
 from .variables import Boolean, Integer, Real, ScalarVariable, String
@@ -22,22 +22,13 @@ FMI2_MODEL_OPTIONS: List[ModelOptions] = [
     ModelOptions("canInterpolateInputs", False, "interpolate-inputs"),
     ModelOptions("canBeInstantiatedOnlyOncePerProcess", False, "only-one-per-process"),
     ModelOptions("canGetAndSetFMUstate", False, "handle-state"),
-    ModelOptions("canSerializeFMUstate", False, "serialize-state"),
-    ModelOptions("canNotUseMemoryManagementFunctions", True, "use-memory-management")
+    ModelOptions("canSerializeFMUstate", False, "serialize-state")
 ]
 
 
 class Fmi2Slave(ABC):
     """Abstract facade class to execute Python through FMI standard."""
 
-    guid: ClassVar[str] = uuid1()
-    author: ClassVar[Optional[str]] = None
-    license: ClassVar[Optional[str]] = None
-    version: ClassVar[Optional[str]] = None
-    copyright: ClassVar[Optional[str]] = None
-    modelName: ClassVar[Optional[str]] = None
-    description: ClassVar[Optional[str]] = None
-
     # Dictionary of (category, description) entries
     log_categories: Dict[str, str] = {
         "logStatusWarning": "Log messages with fmi2Warning status.",
@@ -51,13 +42,17 @@ class Fmi2Slave(ABC):
         self.vars = OrderedDict()
         self.instance_name = kwargs["instance_name"]
         self.resources = kwargs.get("resources", None)
-        self.logger = kwargs.get("logger", None)
         self.visible = kwargs.get("visible", False)
-        self.__lib = None
-        self.__lib_error = False
+        self.log_queue = []
 
-        if self.__class__.modelName is None:
-            self.__class__.modelName = self.__class__.__name__
+        self.guid = uuid1()
+        self.author: Optional[str] = None
+        self.license: Optional[str] = None
+        self.version: Optional[str] = None
+        self.copyright: Optional[str] = None
+        self.modelName: Optional[str] = self.__class__.__name__
+        self.description: Optional[str] = None
+        self.default_experiment: Optional[DefaultExperiment] = None
 
     def to_xml(self, model_options: Dict[str, str] = dict()) -> Element:
         """Build the XML representation of the model.
@@ -98,6 +93,7 @@ class Fmi2Slave(ABC):
             value = model_options.get(option.name, option.value)
             options[option.name] = str(value).lower()
         options["modelIdentifier"] = self.modelName
+        options["canNotUseMemoryManagementFunctions"] = "true"
 
         SubElement(root, "CoSimulation", attrib=options)
 
@@ -111,6 +107,18 @@ class Fmi2Slave(ABC):
                     )
                 )
 
+        if self.default_experiment is not None:
+            attrib = dict()
+            if self.default_experiment.start_time is not None:
+                attrib["startTime"] = str(self.default_experiment.start_time)
+            if self.default_experiment.stop_time is not None:
+                attrib["stopTime"] = str(self.default_experiment.stop_time)
+            if self.default_experiment.step_size is not None:
+                attrib["stepSize"] = str(self.default_experiment.step_size)
+            if self.default_experiment.tolerance is not None:
+                attrib["tolerance"] = str(self.default_experiment.tolerance)
+            SubElement(root, "DefaultExperiment", attrib)
+
         variables = SubElement(root, "ModelVariables")
         for v in self.vars.values():
             if ScalarVariable.requires_start(v):
@@ -158,7 +166,7 @@ class Fmi2Slave(ABC):
         # Set the unique value reference
         var.value_reference = variable_reference
         owner = self
-        if nested and "." in var.name:
+        if var.getter is None and nested and "." in var.name:
             split = var.name.split(".")
             split.pop(-1)
             for s in split:
@@ -296,6 +304,9 @@ class Fmi2Slave(ABC):
     def _fmu_state_from_bytes(state: bytes) -> Dict[str, Any]:
         return json.loads(state.decode("utf-8"))
 
+    def _get_log_queue(self):
+        return self.log_queue
+
     def log(
         self,
         msg: str,
@@ -311,31 +322,9 @@ class Fmi2Slave(ABC):
             category (str or None) : Optional, message category (default derived from status)
             debug (bool) : Optional, is this a debug message (default False)
         """
-        if self.logger is not None and self.resources is not None:
-            if self.__lib is None and not self.__lib_error:
-                library_path = (
-                    Path(self.resources).parent
-                    / "binaries"
-                    / get_platform()
-                    / (self.modelName + "." + get_lib_extension())
-                )
-                try:
-                    self.__lib = cdll.LoadLibrary(str(library_path))
-                except:
-                    self.__lib_error = True
-                    print(
-                        f"Warning. Unable to setup logging for FMU instance: {self.instance_name}"
-                    )
-            if not self.__lib_error:
-                if category is None:
-                    category = f"logStatus{status.name.capitalize()}"
-                    if category not in self.log_categories:
-                        category = "logAll"
-
-                self.__lib.pylog(
-                    c_void_p(self.logger),
-                    c_int(int(status)),
-                    c_char_p(category.encode("utf-8")),
-                    c_char_p(msg.encode("utf-8")),
-                    c_bool(debug)
-                )
+        if category is None:
+            category = f"logStatus{status.name.capitalize()}"
+            if category not in self.log_categories:
+                category = "logAll"
+        log_msg = LogMsg(status, category, msg, debug)
+        self.log_queue.append(log_msg)
